/*  This file is part of libravatarserv.
 *  Copyright © 2018 tastytea <tastytea@tastytea.de>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, version 3.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <iostream>
#include <cstdlib>
#include <basedir.h>
#include "version.hpp"
#include "libravatarserv.hpp"

using std::cout;
using std::cerr;
using std::endl;
using global::avatar_dir;
using global::settings;

// Global variables
std::map<const string, const string> hash::table;
fs::path global::avatar_dir = "";
global::Settings global::settings;

int main()
{
    cout << "Server: libravatarserv/" << global::version << endl;
    cout << "Cache-Control: max-age=86400\n";
    cout << "Connection: close\n";

    const char *request = std::getenv("REQUEST_URI");
    if (request == nullptr)
    {
        cout << "Status: 400 Bad Request\n\n";
        cerr << "Error: ${REQUEST_URI} is empty.\n";
        return 1;
    }
    http::Request avatar = http::parse_request(request);

    if (!find_avatar_dir())
    {
        cout << "Status: 503 Service Unavailable\n\n";
        cerr << "Error: No avatars found.\n";
        return 3;
    }
    hash::fill_table();
    read_settings();

    image::Image image = image::get(avatar.digest, avatar.size);
    if (image.error == 0)
    {
        image::write(image);
    }
    else
    {
        cerr << "Error " << std::to_string(image.error)
             << ": Could not open file.\n";
        if (global::settings.redirect_nouser)
        {
            http::send_redirect(avatar);
            return 0;
        }
        if (avatar.fallback.empty())
        {
            avatar.fallback = settings.default_fallback;
        }
        if (avatar.fallback.substr(0, 3) == "404")
        {
            cout << "Status: 404 Not Found\n\n";
        }
        else if (avatar.fallback.substr(0, 4) == "http")
        {
            cout << "Status: 307 Temporary Redirect\n";
            cout << "Location: " << avatar.fallback << endl << endl;
        }
        else if (avatar.fallback.substr(0, 2) == "mp" ||
                 avatar.fallback.substr(0, 2) == "mm")
        {
            // MD5 hash of 'mp'
            image = image::get("1f2dfa567dcf95833eddf7aec167fec7",
                               avatar.size);
            if (image.error == 0)
            {
                image::write(image);
            }
            else
            {
                cout << "Status: 404 Not Found\n\n";
                cerr << "Error: Mystery person not found.\n";
            }
        }
        else if (avatar.fallback.substr(0, 9) == "identicon")
        {
            image = image::identicon(avatar.digest);
            if (image.error == 0)
            {
                image.image.scale(Magick::Geometry(avatar.size, avatar.size));
                image::write(image);
            }
            else
            {
                cout << "Status: 500 Internal Server Error\n\n";
                cerr << "Error: Couldn't generate identicon.\n";
            }
        }
        else
        {
            if (global::settings.redirect_nofallback)
            {
                http::send_redirect(avatar);
            }
            else
            {
                cout << "Status: 501 Not Implemented\n\n";
            }
        }
    }

    return 0;
}

bool find_avatar_dir()
{
    const char *envdir = std::getenv("LIBRAVATARSERV_DIR");
    if (envdir == nullptr)
    {
        // TODO: Remove AVATAR_DIR in 1.0.0
        // DEPRECATED because it is potentially used by another program
        envdir = std::getenv("AVATAR_DIR");
    }
    if (envdir != nullptr && fs::is_directory(envdir))
    {
        avatar_dir = envdir;
    }
    else
    {
        xdgHandle xdg;
        xdgInitHandle(&xdg);
        auto data_dirs = xdgDataDirectories(&xdg);
        const uint8_t size = sizeof(data_dirs) / sizeof(*data_dirs);

        for (uint8_t index = 0; index <= size; ++index)
        {
            const string searchdir = data_dirs[index]
                                   + string("/libravatarserv");
            if (fs::is_directory(searchdir))
            {
                avatar_dir = searchdir;
                break;
            }
        }
        xdgWipeHandle(&xdg);
        if (avatar_dir.empty())
        {
            return false;
        }
    }

    return true;
}

void read_settings()
{
    const char *env = std::getenv("LIBRAVATARSERV_DEFAULT_FALLBACK");
    if (env != nullptr)
    {
        global::settings.default_fallback = env;
    }

    env = std::getenv("LIBRAVATARSERV_REDIRECT_NOFALLBACK");
    if (env != nullptr && env[0] == '1')
    {
        global::settings.redirect_nofallback = true;
    }

    env = std::getenv("LIBRAVATARSERV_REDIRECT_NOUSER");
    if (env != nullptr && env[0] == '1')
    {
        global::settings.redirect_nouser = true;
    }
}
